unit MakeX509CertMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, MPDL, MPIF, ASN1;

type
  TfrmMakeCertMain = class(TForm)
    btnMakeCACert: TButton;
    rgShowCACert: TRadioGroup;
    memCACert: TMemo;
    rgCAKey: TRadioGroup;
    rgKey: TRadioGroup;
    btnMakeCert: TButton;
    rgShowCert: TRadioGroup;
    memCert: TMemo;
    Button1: TButton;
    Button2: TButton;
    procedure FormDestroy(Sender: TObject);
    procedure btnMakeCACertClick(Sender: TObject);
    procedure rgShowCACertClick(Sender: TObject);
    procedure btnMakeCertClick(Sender: TObject);
    procedure rgShowCertClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    FDLCAPriv: TDLPrivateKey;
    FIFCAPriv: TIFPrivateKey;  
    FDLCAPubl: TDLPublicKey;
    FIFCAPubl: TIFPublicKey;
    FCACert: TASN1Struct;      
    FCert: TASN1Struct;
  public
    { Public declarations }
  end;

var
  frmMakeCertMain: TfrmMakeCertMain;

implementation

uses
  SecUtils, PKIX, X509;

{$R *.DFM}

procedure TfrmMakeCertMain.FormDestroy(Sender: TObject);
begin
  DisposeDLPrivateKey(FDLCAPriv);
  DisposeIFPrivateKey(FIFCAPriv);
  DisposeDLPublicKey(FDLCAPubl);
  DisposeIFPublicKey(FIFCAPubl);
  FCACert.Free;
  FCert.Free;
end;

procedure TfrmMakeCertMain.btnMakeCACertClick(Sender: TObject);
var
  PK: TX509PublicKey;
  Subject: TX501Name;
  Validity: TX509Validity;
  AltName: TX509GeneralNames;
  DP: TX509CRLDistPoints;
begin
  DisposeDLPrivateKey(FDLCAPriv);
  DisposeIFPrivateKey(FIFCAPriv);
  DisposeDLPublicKey(FDLCAPubl);
  DisposeIFPublicKey(FIFCAPubl);

  PK := TX509PublicKey.Create;
  try
    if rgCAKey.ItemIndex = 0 then begin
      IFRSAKeys(1024,1,False,FIFCAPriv,FIFCAPubl);
      TranslateRSAPublicKey(FIFCAPubl,PK);
    end else {if rgCAKey.ItemIndex = 1 then} begin
      DLKeys(1024,160,FDLCAPriv,FDLCAPubl);
      TranslateDSAPublicKey(FDLCAPubl,PK);
    end;

    Subject.commonName.Str := 'Big Boss';
    Subject.surname.Str := 'Boss';
    Subject.countryName.Str := 'US';
    Subject.organizationName.Str := 'ACME Inc';
    Subject.title.Str := 'Mr';
    Subject.initials.Str := 'BB';
    Subject.givenName.Str := 'Big';

    Validity.notBefore := Now;
    Validity.notAfter := Now + 365;

    SetLength(AltName,2);
    AltName[0].Tag := 1;
    AltName[0].rfc822Name := 'big.boss@acme.com';
    AltName[1].Tag := 6;
    AltName[1].uniformResourceIdentifier := 'http://www.acme.com/bigboss/';

    SetLength(DP,1);
    DP[0].distributionPoint.Tag := 0;
    SetLength(DP[0].distributionPoint.fullName,1);
    DP[0].distributionPoint.fullName[0].Tag := 6;
    DP[0].distributionPoint.fullName[0].uniformResourceIdentifier := 'http://www.acme.com/bigboss/recent.crl';
    DP[0].reasons := [LowReasonFlag..High(TX509ReasonFlag)];
    DP[0].CRLIssuer := nil;

    ComposeCACertificate2(FCACert,                               // Output
                          Subject,                               // Subject Name
                          PK,                                    // Subject Public Key
                          Validity,                              // notBefore/notAfter
                          [digitalSignature,keyCertSign,crlSign],// Key Usage
                          nil,                                   // Extended Key Usage
                          AltName,                               // Subject Alternative Name
                          DP,                                    // CRL Distribution points
                          0,                                     // Maximum length of CA chain, 0 means only end user certificates
                          '01');                                 // Serial number

    if rgCAKey.ItemIndex = 0 then
      RSASignCertificate(FCACert,FCACert,FIFCAPriv,haSHA1,haSHA1,seEMSA3)
    else
      DSASignCertificate(FCACert,FCACert,FDLCAPriv);

    rgShowCACertClick(Sender);

    btnMakeCert.Enabled := True;
  finally
    PK.Free;
  end;
end;

procedure TfrmMakeCertMain.rgShowCACertClick(Sender: TObject);
var
  SS: TStringStream;
begin
  if Assigned(FCACert) then begin
    memCACert.Clear;
    if rgShowCACert.ItemIndex = 0 then
      RenderAsText(FCACert,memCACert.Lines,True,True,True,0,80)
    else begin
      SS := TStringStream.Create('');
      try
        FCACert.SaveToStream(SS,fmtDOM);
        memCACert.Text := SS.DataString;
      finally
        SS.Free;
      end;
    end;
  end;
end;

procedure TfrmMakeCertMain.btnMakeCertClick(Sender: TObject);
var
  DLPubl: TDLPublicKey;
  IFPubl: TIFPublicKey;
  DLPriv: TDLPrivateKey;
  IFPriv: TIFPrivateKey;

  PK: TX509PublicKey;
  Subject: TX501Name;
  Validity: TX509Validity;
  AltName: TX509GeneralNames;
  KeyUsage: TKeyUsage;
  ExtKeyUsage: TStrings;
begin
  FillChar(DLPubl,SizeOf(DLPubl),0);
  FillChar(IFPubl,SizeOf(IFPubl),0); 
  FillChar(DLPriv,SizeOf(DLPriv),0);
  FillChar(IFPriv,SizeOf(IFPriv),0);
  try

    PK := TX509PublicKey.Create;
    try
      if rgKey.ItemIndex = 0 then begin
        IFRSAKeys(1024,1,False,IFPriv,IFPubl);
        TranslateRSAPublicKey(IFPubl,PK);
        KeyUsage := [digitalSignature,keyEncipherment,encipherOnly];
      end else if rgKey.ItemIndex = 1 then begin
        DLKeys(1024,160,DLPriv,DLPubl);
        TranslateDSAPublicKey(DLPubl,PK); 
        KeyUsage := [digitalSignature];
      end else begin
        DLKeys(1024,160,DLPriv,DLPubl);
        TranslateDHPublicKey(DLPubl,PK);
        KeyUsage := [keyAgreement,encipherOnly];
      end;

      Subject.commonName.Str := 'John Doe';
      Subject.surname.Str := 'Doe';
      Subject.countryName.Str := 'US';
      Subject.organizationName.Str := 'ACME Inc';
      Subject.title.Str := 'Mr';
      Subject.initials.Str := 'JD';
      Subject.givenName.Str := 'John';

      Validity.notBefore := Now;
      Validity.notAfter := Now + 365;

      SetLength(AltName,2);
      AltName[0].Tag := 1;
      AltName[0].rfc822Name := 'john.doe@acme.com';

      ExtKeyUsage := TStringList.Create;
      try
        ExtKeyUsage.Add(id_kp_emailProtection);

        ComposeCertificate2(FCert,                                 // Output
                            Subject,                               // Subject Name
                            PK,                                    // Subject Public Key
                            FCACert,                               // CA Certificate
                            Validity,                              // notBefore/notAfter
                            KeyUsage,                              // Key Usage
                            ExtKeyUsage,                           // Extended Key Usage
                            AltName,                               // Subject Alternative Name
                            '02');                                 // Serial number
      finally
        ExtKeyUsage.Free;
      end;

      if rgCAKey.ItemIndex = 0 then
        RSASignCertificate(FCert,FCACert,FIFCAPriv,haSHA1,haSHA1,seEMSA3)
      else
        DSASignCertificate(FCert,FCACert,FDLCAPriv);

      rgShowCertClick(Sender);
    finally
      PK.Free;
    end;
  finally
    DisposeDLPublicKey(DLPubl);
    DisposeIFPublicKey(IFPubl);
    DisposeDLPrivateKey(DLPriv);
    DisposeIFPrivateKey(IFPriv);
  end;
end;

procedure TfrmMakeCertMain.rgShowCertClick(Sender: TObject);
var
  SS: TStringStream;
begin
  if Assigned(FCert) then begin
    memCert.Clear;
    if rgShowCert.ItemIndex = 0 then
      RenderAsText(FCert,memCert.Lines,True,True,True,0,80)
    else begin
      SS := TStringStream.Create('');
      try
        FCert.SaveToStream(SS,fmtDOM);
        memCert.Text := SS.DataString;
      finally
        SS.Free;
      end;
    end;
  end;
end;

procedure TfrmMakeCertMain.Button1Click(Sender: TObject);
var
  MS, MS0: TMemoryStream;
begin
  FCACert.AllowResume := True;
  MS := TMemoryStream.Create;
  try                          
    FCACert.ResetStreaming;
    FCACert.SaveToStream(MS,fmtDER);
    MS.Position := 0;
    MS0 := TMemoryStream.Create;
    try
      MS0.CopyFrom(MS,MS.Size shr 1);
      MS0.Position := 0;
//      FCACert.Clear;
      FCACert.AllowResume := True;
      FCACert.LoadFromStream(MS0,fmtDER);
      rgShowCACertClick(Sender);
      FCACert.LoadFromStream(MS,fmtDER);
    finally
      MS0.Free;
    end;
  finally
    MS.Free;
  end;
end;

procedure TfrmMakeCertMain.Button2Click(Sender: TObject);
var
  MS: TMemoryStream;
begin
  MS := TMemoryStream.Create;
  try
    FCACert.ResetStreaming;
    FCACert.SaveToStream(MS,fmtDER);
    MS.Position := 0;
    FCACert.ResetStreaming;
    FCACert.AllowResume := True;
    FCACert.LoadFromStream(MS,fmtDER);
    rgShowCACertClick(Sender);
  finally
    MS.Free;
  end;
end;

end.
